# CNAs with HTML advisories

## Adobe Systems Incorporated
### Advisory
https://helpx.adobe.com/security/alertus.html 
#### Notes
Advisories are located here: https://helpx.adobe.com/security/security-bulletin.html. This page can be used as an index to get all individual adivsory ids.
Each advisory has HTML tables with tr and td tags. h1 and h2 tags for validation of specific headings.
[adobe_security_bulletin_example](img/adobe_security_bulletin_example.png)
#### Captured Data Potential From Advisory
```
{
  :bulletin_id => 'APSB22-21',
  :date_published => 'May 10, 2022',
  :priority => '3',
  :summary => 'Adobe has released an update for Adobe Character Animator for Windows and macOS. This update resolves a critical vulnerability. Successful exploitation could lead to arbitrary code execution' ,
  :affected_versions => [{:product => 'Character Animator 2021', :version => '4.4.2 and earlier versions', :platform => 'Windows and macOS'}, {:product => 'Character Animator 2022', :version => '22.3 and earlier versions', :platform => 'Windows and macOS'}],
  :solution => [{:product => 'Character Animator 2021', :version => '4.4.7', :platform => 'Windows and macOS', :priority_rating => '3', :availability => 'https://www.adobe.com/creativecloud/catalog/desktop.html' }, {:product => 'Character Animator 2022', :version => '22.4', :platform => 'Windows and macOS', :availability => 'https://www.adobe.com/creativecloud/catalog/desktop.html'}],
  :vulnerability_details => [{:vulnerability_category => 'Out-of-bounds Write (CWE-787)', :vulnerability_impact => 'Arbitrary code execution', :severity => 'Critical', :cvss_base_score => '7.8', :cvss_vector => 'CVSS:3.0/AV:L/AC:L/PR:N/UI:R/S:U/C:H/I:H/A:H', :cve_numbers => 'CVE-2022-28819'}]
}
```

## AMD
### Advisory
https://www.amd.com/en/corporate/product-security
#### Captured Data Potential From Advisory
```
# Note: Additional advisories have some more keys than the example shown below
# for example: AMD-SB-1028 also has a :cve_details table with :cve, :cvss_score, :cve_description in each row. 
{
  :bulletin_id => 'AMD-SB-1033',
  :potential_impact => 'Data leakage',
  :severity =>'Medium',
  :summary => 'AMD received notification of a potential security vulnerability where data in specific cryptographic algorithms can be inferred in a SEV guest by monitoring the ciphertext values over time.  Researchers from the Institute for IT-Security University of Luebeck and The Ohio State University will present their paper titled “A Systematic Look at Ciphertext Side Channels on AMD SEV-SNP” at the 43rd IEEE Symposium on Security and Privacy scheduled for May 23-25, 2022. CVE-2021-46744 An attacker with access to a malicious hypervisor may be able to infer data values used in a SEV guest on AMD CPUs by monitoring ciphertext values over time.',
  :affected_products => ['1st/2nd/3rdGen AMD EPYC™ Processors', 'AMD EPYC™ Embedded Processors'],
  :mitigation => 'AMD has provided guidance via an updated paper located here: TECHNICAL GUIDANCE FOR MITIGATING EFFECTS OF CIPHERTEXT VISIBILITY UNDER AMD SEV As of the date of this notice, AMD is not aware of any active exploits in the wild of CVE-2021-46744 on AMD products. We continue to look for ways to make our products more secure, including working closely with partners, academics, researchers, and end-users in the ecosystem.',
  :references => 'https://www.amd.com/system/files/documents/221404394-a_security_wp_final.pdf'
}
```

## Ampere
#### Advisory
https://amperecomputing.com/products/product-security.html
#### Captured Data Potential From Advisory
```
{
  :bulletin_id => 'AMP-SB-0001',
  :potential_impact => 'Partial loss of protected data',
  :severity => 'Medium: CVSS score is 4.7',
  :summary => '[CVSS:3.1/AV:L/AC:H/PR:L/UI:N/S:U/C:N/I:H/A:N] 3/8/2022: Ampere investigated the potential impacts of the Spectre-BHB to Ampere products. The description of the vulnerability, published by the national vulnerability database (NVD) states: CVE-2022-25368 Spectre BHB is a variant of Spectre-v2 in which malicious code uses the shared branch history (stored in the CPU BHB) to influence mispredicted branches in the victim's hardware context. Speculation caused by these mispredicted branches can then potentially be used to cause cache allocation, which can then be used to infer information that should be protected.',
  :affected_products => 'Ampere® Altra® and Ampere® Altra® Max',
  :recommendations => 'Ampere recommends that users of affected products update to the latest firmware version provided by the system manufacturer that addresses these issues. Ampere also recommends updating to the latest OS updates that that addresses these issues. Disable unprivileged Extended Berkeley Packet Filter (eBPF)',
  :references => [
    'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2022-25368',
    'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2022-23960',
    'https://developer.arm.com/support/arm-security-updates/speculative-processor-vulnerability/spectre-bhb'
  ]
}
```

## Android
### Advisory
https://source.android.com/security/bulletin
#### Notes
We need to figure out a way to parse these google bulletins. Headings and tables. the CVE data is in the tables.
#### Captured Data Potential From Advisory

## Apple Inc.
### Advisory
https://support.apple.com/en-us/HT201222
#### Captured Data Potential From Advisory
```
[
  {
    "cves": [
      {
        "cve_id": "CVE-2017-2372",
        "cve_author": "Tyler Bohan of Cisco Talos"
      }
    ],
    "name":
      {
        "name": "Projects"
      },
    "available_for": "OS X El Capitan v10.11 and later (64 bit)",
    "description": "A memory corruption issue was addressed through improved memory handling.",
    "impact": "Opening a maliciously crafted GarageBand project file may lead to arbitrary\u00a0code execution"
  }
]
```

## Arista Networks
### Advisory
https://www.arista.com/en/support/advisories-notices 
#### Notes
Offers a subscribe with RSS button to get an index of advisories at least.
#### Captured Data Potential From Advisory
```
{
  :bulletin_id => 'Security Advisory 0068',
  :date => 'August 20th, 2021',
  :version => '1.0',
  :revision => [{:version => '1.0', :date => 'August 20th, 2021', :changes => 'Initial Release'}],
  :cve_id => 'CVE-2021-28494',
  :cvss_base_score => '9.6( CVSS:3.1/AV:N/AC:L/PR:L/UI:N/S:C/C:N/I:H/A:H)',
  :description => 'This advisory documents the impact of an internally found vulnerability in Arista's MOS (Metamako Operating System) software which is supported on the 7130 product line. The effect of this vulnerability is that, under certain conditions, authentication is bypassed by unprivileged users who are accessing the Web UI. This issue was discovered internally and Arista is not aware of any malicious uses of this issue in customer networks.',
  :vulnerability_assessment => {
    :affected_software => 'MOS-0.34.0 and prior releases',
    :affected_platforms => { 
      :affected_products => ['Arista 7130 Systems running MOS'], 
      :non_affected_products => [
        'Arista EOS-based products',
        'Arista Converged Cloud Fabric and DANZ Monitoring Fabric (Formerly Big Switch Nodes for BCF and BMF)',
        'Arista Wireless Access Points',
        'CloudVision Wi-Fi (on-premise and cloud service delivery)',
        'CloudVision Portal, virtual appliance or physical appliance',
        'CloudVision as-a-Service',
        'Awake Security Platform'
      ]}},
  :symptoms => 'Check if the installed software version falls under the list specified in the “Affected Software” section:',
  :mitigation => 'Web UI access to the affected systems is enabled by default and can be disabled by using the following commands. However, note that this will break any automation that relies on Web UI or JSON API access to the system and should be used only if Web UI and JSON API access is not required for managing the system.',
  :resolution => 'This vulnerability is being tracked by BUG567401. The recommended resolution is to upgrade to a remediated MOS version during a maintenance window. This vulnerability has been fixed in the following MOS version: MOS-0.35.0',
}
```

## Atlassian
### Advisory
https://www.atlassian.com/trust/security/advisories 
#### Notes
The page listed in the CNAList.json for this org is the advisory policy document and not the list of advisories. The link provided in this document is the correct link for security advisories
#### Captured Data Potential From Advisory
```
{
  :bulletin_id => 'Jira Security Advisory 2022-04-20',
  :summary_table => {
    :summary => 'CVE-2022-0540 - Authentication bypass in Seraph',
    :advisory_release_date => '20 Apr 2022 10:00 AM PDT (Pacific Time, -7 hours)',
    :affected_products => [
      'Jira', 
      'Jira Core Server',
      'Jira Software Server',
      'Jira Software Data Center',
      'Jira Service Management',
      'Jira Service Management Server',
      'Jira Service Management Data Center'
    ],
    :cve_ids => 'CVE-2022-0540'
  },
  :summary_of_vulnerability => 'Jira and Jira Service Management are vulnerable to an authentication bypass in its web authentication framework, Jira Seraph. Although the vulnerability is in the core of Jira, it affects first and third party apps that specify roles-required at the webwork1 action namespace level and do not specify it at an action level. For a specific action to be affected, the action will also need to not perform any other authentication or authorization checks. A remote, unauthenticated attacker could exploit this by sending a specially crafted HTTP request to bypass authentication and authorization requirements in WebWork actions using an affected configuration.',
  :severity => 'For installations that use apps that have an affected configuration, Atlassian rates the severity level of this vulnerability as critical, though this may vary if an affected app uses additional permissions checks. For more detailed information on the impact to each app listed in the Determining which apps are affected section below, contact the app vendor. For installations that do not use any apps that have an affected configuration as described in the Summary of Vulnerability section above, Atlassian rates the severity level of this vulnerability as medium. This is our assessment, and you should evaluate its applicability to your own IT environment.',
  :affected_jira_versions => [
    'Jira Core Server',
    'Jira Software Server',
    'Jira Software Data Center',
    'All versions before 8.13.18',
    '8.14.x',
    '8.15.x',
    '8.16.x',
    '8.17.x',
    '8.18.x',
    '8.19.x',
    '8.20.x before 8.20.6',
    '8.21.x'
  ],
  :fixed_jira_versions => [
    '8.13.x >= 8.13.18',
    '8.20.x >= 8.20.6',
    'All versions >= 8.22.0'
  ],
  :fixed_jira_service_management_versions => [
    '4.13.x >= 4.13.18',
    '4.20.x >= 4.20.6',
    'All versions >= 4.22.0',
  ]
}
```

## Autodesk
### Advisory
https://www.autodesk.com/trust/security-advisories
#### Captured Data Potential From Advisory

## Avaya, Inc
### Advisory
https://support.avaya.com/security
#### Captured Data Potential From Advisory

## Becton, Dickinson and Company (BD)
### Advisory
https://cybersecurity.bd.com/bulletins-and-patches
#### Captured Data Potential From Advisory

## Bitdefender
### Advisory
https://www.bitdefender.com/support/security-advisories/
#### Captured Data Potential From Advisory

## BlackBerry
### Advisory
https://www.blackberry.com/us/en/services/blackberry-product-security-incident-response
#### Captured Data Potential From Advisory

## Brocade Communication Systems, LLC
### Advisory
https://www.broadcom.com/support/fibre-channel-networking/security-advisories
#### Captured Data Potential From Advisory

## CA Technologies - A Broadcom Company
### Advisory
- Software advisories: https://support.broadcom.com/security-advisory/security-advisories-list.html?segment=ES
- Mainframe advisories: https://support.broadcom.com/security-advisory/security-advisories-list.html?segment=MF
#### Notes
Both pages need an account to login to view advisories
#### Captured Data Potential From Advisory

## CERT/CC
### Advisory
https://www.kb.cert.org/vuls/bypublished/desc/
#### Captured Data Potential From Advisory

## CERT@VDE
### Advisory
https://cert.vde.com/en/advisories/
#### Captured Data Potential From Advisory

## Checkpoint
### Advisory
https://www.checkpoint.com/advisories/
#### Captured Data Potential From Advisory

## Chrome
### Advisory
https://www.google.com/about/appsecurity/research/
#### Notes
CNA json lists https://www.google.com/about/appsecurity/research/ which redirects to their bughutner portal. Navigating from there to this page shows some public reports: https://bughunters.google.com/report/reports
#### Captured Data Potential From Advisory

## CISA - ICS
### Advisory
https://www.cisa.gov/uscert/ics/alerts 
#### Notes
They offer an rss feed to get the index of advisories
#### Captured Data Potential From Advisory

## Cyber Security Works Pvt. Ltd.
### Advisory
https://cybersecurityworks.com/zerodays-vulnerability-list/
#### Notes
Paginated list of vulns. 
#### Captured Data Potential From Advisory

## Dahua Technologies
### Advisory
https://www.dahuasecurity.com/support/cybersecurity/annoucementNotice
#### Notes
Paginated index of vulns.
#### Captured Data Potential From Advisory

## Debian Gnu/Linux
### Advisory
https://www.debian.org/security/#DSAS
#### Notes
Wide range of formats.
#### Captured Data Potential From Advisory

## DeepSurface Security, Inc
### Advisory
https://deepsurface.com/tag/blog/
#### Notes
The page has advisories but also just regular release notes and blog posts
#### Captured Data Potential From Advisory

## Dell
### Advisory
https://www.dell.com/support/security/en-us
### Notes
Has an Export table button that can be used to export the index of cves to a spreadsheet
#### Captured Data Potential From Advisory

## Devolutions Inc.
### Advisory
https://devolutions.net/security/advisories/
#### Notes
HTML table for index. subscribe to security advisories via email on bottom of index page.
#### Captured Data Potential From Advisory

## Document Foundatino, The
### Advisory
https://www.libreoffice.org/about-us/security/advisories/
#### Notes
Header and p tags for index
#### Captured Data Potential From Advisory

## Drupal.org
### Advisory
https://www.drupal.org/security
#### Notes
Heading and p tags for index
#### Captured Data Potential From Advisory

## Elastic
### Advisory
https://www.elastic.co/community/security
#### Notes
HTML table index has everything. cve-id links to mitre. ESA-ID, CVE, Date Disclosed, Vulnerability Summary, Remediation Summary
#### Captured Data Potential From Advisory
```
{ 
  :esa_id => "ESA-2021-09",
  :cve => "CVE-2021-22138", 
  :date_disclosed => "2021-03-23",
  :vulnerability_summary => "A TLS certificate validation flaw was found in the monitoring feature of Logstash versions 6.4.0 and before versions 6.8.15 and 7.12.0. When specifying a trusted server CA certificate Logstash would not properly verify the certificate returned by the monitoring server. This could result in a man in the middle style attack against the Logstash monitoring data.",
  :remediation_summary => "Users should update their version of Logstash to 7.12.0 or 6.8.15."
}
```